import copy
import http.client as httplib
import logging
import multiprocessing
import platform
import sys
from logging import FileHandler
from typing import Any, Dict, Optional, Union

from typing_extensions import Self

from zitadel_client.auth.authenticator import Authenticator
from zitadel_client.version import Version


class Configuration:
    """This class contains various settings of the API client."""

    USER_AGENT = " ".join(
        [
            f"zitadel-client/{Version.VERSION}",
            "("
            + "; ".join(
                [
                    "lang=python",
                    f"lang_version={platform.python_version()}",
                    f"os={platform.system()}",
                    f"arch={platform.machine()}",
                ]
            )
            + ")",
        ]
    ).lower()

    def __init__(
        self,
        authenticator: Authenticator,
        ssl_ca_cert: Optional[str] = None,
        retries: Optional[int] = None,
        ca_cert_data: Optional[Union[str, bytes]] = None,
        *,
        debug: Optional[bool] = None,
    ) -> None:
        """Constructor"""
        self._user_agent = Configuration.USER_AGENT
        self.authenticator = authenticator
        self.refresh_api_key_hook = None
        self.logger = {
            "package_logger": logging.getLogger("zitadel_client"),
            "urllib3_logger": logging.getLogger("urllib3"),
        }
        self.logger_format = "%(asctime)s %(levelname)s %(message)s"
        self.logger_stream_handler = None
        self.logger_file_handler: Optional[FileHandler] = None
        self.logger_file = None
        if debug is not None:
            self.debug = debug
        else:
            self.__debug = False
        self.verify_ssl = True
        self.ssl_ca_cert = ssl_ca_cert
        self.ca_cert_data = ca_cert_data
        self.cert_file = None
        self.key_file = None
        self.assert_hostname = None
        self.tls_server_name = None

        # noinspection PyUnresolvedReferences
        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5
        """urllib3 connection pool's maximum number of connections saved
       per pool. urllib3 uses 1 connection as default value, but this is
       not the best value when you are making a lot of possibly parallel
       requests to the same host, which is often the case here.
       cpu_count * 5 is used as default value to increase performance.
    """

        self.safe_chars_for_path_param = ""
        self.retries = retries
        # Enable client side validation
        self.client_side_validation = True
        self.socket_options = None
        self.datetime_format = "%Y-%m-%dT%H:%M:%S.%f%z"
        self.date_format = "%Y-%m-%d"

    def __deepcopy__(self, memo: Dict[int, Any]) -> Self:
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        for k, v in self.__dict__.items():
            if k not in ("logger", "logger_file_handler"):
                # noinspection PyArgumentList
                setattr(result, k, copy.deepcopy(v, memo))
        # shallow copy of loggers
        result.logger = copy.copy(self.logger)
        # use setters to configure loggers
        result.logger_file = self.logger_file
        result.debug = self.debug
        return result

    def __setattr__(self, name: str, value: Any) -> None:
        object.__setattr__(self, name, value)

    @property
    def logger_file(self) -> Optional[str]:
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :type: str
        """
        return self.__logger_file

    @logger_file.setter
    def logger_file(self, value: Optional[str]) -> None:
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        self.__logger_file = value
        if self.__logger_file:
            # If set logging file,
            # then add file handler and remove stream handler.
            self.logger_file_handler = logging.FileHandler(self.__logger_file)
            self.logger_file_handler.setFormatter(self.logger_formatter)
            for _, logger in self.logger.items():
                logger.addHandler(self.logger_file_handler)

    @property
    def debug(self) -> bool:
        """Debug status

        :type: bool
        """
        return self.__debug

    @debug.setter
    def debug(self, value: bool) -> None:
        """Debug status

        :param value: The debug status, True or False.
        :type: bool
        """
        self.__debug = value
        if self.__debug:
            # if debug status is True, turn on debug logging
            for _, logger in self.logger.items():
                logger.setLevel(logging.DEBUG)
            # turn on httplib debug
            httplib.HTTPConnection.debuglevel = 1
        else:
            # if debug status is False, turn off debug logging,
            # setting log level to default `logging.WARNING`
            for _, logger in self.logger.items():
                logger.setLevel(logging.WARNING)
            # turn off httplib debug
            httplib.HTTPConnection.debuglevel = 0

    @property
    def logger_format(self) -> str:
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :type: str
        """
        return self.__logger_format

    @logger_format.setter
    def logger_format(self, value: str) -> None:
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        self.__logger_format = value
        self.logger_formatter = logging.Formatter(self.__logger_format)

    @staticmethod
    def to_debug_report() -> str:
        """Gets the essential information for debugging.

        :return: The report for debugging.
        """
        return (
            "Python SDK Debug Report:\n"
            "OS: {env}\n"
            "Python Version: {pyversion}\n"
            "Version of the API: 1.0.0\n"
            "SDK Package Version: 0.0.1".format(env=sys.platform, pyversion=sys.version)
        )

    @property
    def host(self) -> str:
        """Return generated host."""
        return self.authenticator.get_host()

    @property
    def user_agent(self) -> str:
        """
        Get the user agent string.

        Returns:
            str: The current value of the user agent.
        """
        return self._user_agent

    @user_agent.setter
    def user_agent(self, value: str) -> None:
        """
        Set the user agent string.

        Args:
            value (str): The new user agent string to set.
        """
        self._user_agent = value



from abc import ABC, abstractmethod
from datetime import datetime, timedelta, timezone
from typing import Any, Dict, Generic, TypeVar  # noqa: F401


class Authenticator(ABC):
    """
    Abstract base class for authenticators.

    This class defines the basic structure for any authenticator by requiring the implementation
    of a method to retrieve authentication headers, and provides a way to store and retrieve the host.
    """

    def __init__(self, host: str):
        """
        Initializes the Authenticator with the specified host.

        :param host: The base URL or endpoint for the service.
        """
        self.host = host

    @abstractmethod
    def get_auth_headers(self) -> Dict[str, str]:
        """
        Retrieves the authentication headers to be sent with requests.

        Subclasses must override this method to return the appropriate headers.

        :return: A dictionary mapping header names to their values.
        """
        pass  # pragma: no cover

    def get_host(self) -> str:
        """
        Returns the stored host.

        :return: The host as a string.
        """
        return self.host


class Token:
    def __init__(self, access_token: str, expires_at: datetime):
        """
        Initializes a new Token instance.

        Parameters:
        - access_token (str): The JWT or OAuth token.
        - expires_at (datetime): The expiration time of the token. It should be timezone-aware.
          If a naive datetime is provided, it will be converted to an aware datetime in UTC.
        """
        self.access_token = access_token

        # Ensure expires_at is timezone-aware. If naive, assume UTC.
        if expires_at.tzinfo is None:
            self.expires_at = expires_at.replace(tzinfo=timezone.utc)
        else:
            self.expires_at = expires_at

    def is_expired(self) -> bool:
        """
        Checks if the token is expired by comparing the current UTC time
        with the token's expiration time.

        Returns:
        - bool: True if expired, False otherwise.
        """
        return datetime.now(timezone.utc) >= (self.expires_at - timedelta(minutes=5))



# coding: utf-8

"""
    Zitadel SDK

    The Zitadel SDK is a convenience wrapper around the Zitadel APIs to assist you in integrating with your Zitadel environment. This SDK enables you to handle resources, settings, and configurations within the Zitadel platform.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, Optional
from zitadel_client.models.beta_authorization_service_id_filter import BetaAuthorizationServiceIDFilter
from zitadel_client.models.beta_authorization_service_in_ids_filter import BetaAuthorizationServiceInIDsFilter
from zitadel_client.models.beta_authorization_service_project_name_query import BetaAuthorizationServiceProjectNameQuery
from zitadel_client.models.beta_authorization_service_role_key_query import BetaAuthorizationServiceRoleKeyQuery
from zitadel_client.models.beta_authorization_service_state_query import BetaAuthorizationServiceStateQuery
from zitadel_client.models.beta_authorization_service_user_display_name_query import BetaAuthorizationServiceUserDisplayNameQuery
from zitadel_client.models.beta_authorization_service_user_preferred_login_name_query import BetaAuthorizationServiceUserPreferredLoginNameQuery
from typing import Optional, Set
from typing_extensions import Self

class BetaAuthorizationServiceAuthorizationsSearchFilter(BaseModel):
    """
    BetaAuthorizationServiceAuthorizationsSearchFilter
    """ # noqa: E501
    authorization_ids: Optional[BetaAuthorizationServiceInIDsFilter] = Field(default=None, alias="authorizationIds")
    organization_id: Optional[BetaAuthorizationServiceIDFilter] = Field(default=None, alias="organizationId")
    project_grant_id: Optional[BetaAuthorizationServiceIDFilter] = Field(default=None, alias="projectGrantId")
    project_id: Optional[BetaAuthorizationServiceIDFilter] = Field(default=None, alias="projectId")
    project_name: Optional[BetaAuthorizationServiceProjectNameQuery] = Field(default=None, alias="projectName")
    role_key: Optional[BetaAuthorizationServiceRoleKeyQuery] = Field(default=None, alias="roleKey")
    state: Optional[BetaAuthorizationServiceStateQuery] = None
    user_display_name: Optional[BetaAuthorizationServiceUserDisplayNameQuery] = Field(default=None, alias="userDisplayName")
    user_id: Optional[BetaAuthorizationServiceIDFilter] = Field(default=None, alias="userId")
    user_organization_id: Optional[BetaAuthorizationServiceIDFilter] = Field(default=None, alias="userOrganizationId")
    user_preferred_login_name: Optional[BetaAuthorizationServiceUserPreferredLoginNameQuery] = Field(default=None, alias="userPreferredLoginName")
    __properties: ClassVar[List[str]] = ["authorizationIds", "organizationId", "projectGrantId", "projectId", "projectName", "roleKey", "state", "userDisplayName", "userId", "userOrganizationId", "userPreferredLoginName"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BetaAuthorizationServiceAuthorizationsSearchFilter from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of authorization_ids
        if self.authorization_ids:
            _dict['authorizationIds'] = self.authorization_ids.to_dict()
        # override the default output from pydantic by calling `to_dict()` of organization_id
        if self.organization_id:
            _dict['organizationId'] = self.organization_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of project_grant_id
        if self.project_grant_id:
            _dict['projectGrantId'] = self.project_grant_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of project_id
        if self.project_id:
            _dict['projectId'] = self.project_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of project_name
        if self.project_name:
            _dict['projectName'] = self.project_name.to_dict()
        # override the default output from pydantic by calling `to_dict()` of role_key
        if self.role_key:
            _dict['roleKey'] = self.role_key.to_dict()
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict['state'] = self.state.to_dict()
        # override the default output from pydantic by calling `to_dict()` of user_display_name
        if self.user_display_name:
            _dict['userDisplayName'] = self.user_display_name.to_dict()
        # override the default output from pydantic by calling `to_dict()` of user_id
        if self.user_id:
            _dict['userId'] = self.user_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of user_organization_id
        if self.user_organization_id:
            _dict['userOrganizationId'] = self.user_organization_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of user_preferred_login_name
        if self.user_preferred_login_name:
            _dict['userPreferredLoginName'] = self.user_preferred_login_name.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BetaAuthorizationServiceAuthorizationsSearchFilter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "authorizationIds": BetaAuthorizationServiceInIDsFilter.from_dict(obj["authorizationIds"]) if obj.get("authorizationIds") is not None else None,
            "organizationId": BetaAuthorizationServiceIDFilter.from_dict(obj["organizationId"]) if obj.get("organizationId") is not None else None,
            "projectGrantId": BetaAuthorizationServiceIDFilter.from_dict(obj["projectGrantId"]) if obj.get("projectGrantId") is not None else None,
            "projectId": BetaAuthorizationServiceIDFilter.from_dict(obj["projectId"]) if obj.get("projectId") is not None else None,
            "projectName": BetaAuthorizationServiceProjectNameQuery.from_dict(obj["projectName"]) if obj.get("projectName") is not None else None,
            "roleKey": BetaAuthorizationServiceRoleKeyQuery.from_dict(obj["roleKey"]) if obj.get("roleKey") is not None else None,
            "state": BetaAuthorizationServiceStateQuery.from_dict(obj["state"]) if obj.get("state") is not None else None,
            "userDisplayName": BetaAuthorizationServiceUserDisplayNameQuery.from_dict(obj["userDisplayName"]) if obj.get("userDisplayName") is not None else None,
            "userId": BetaAuthorizationServiceIDFilter.from_dict(obj["userId"]) if obj.get("userId") is not None else None,
            "userOrganizationId": BetaAuthorizationServiceIDFilter.from_dict(obj["userOrganizationId"]) if obj.get("userOrganizationId") is not None else None,
            "userPreferredLoginName": BetaAuthorizationServiceUserPreferredLoginNameQuery.from_dict(obj["userPreferredLoginName"]) if obj.get("userPreferredLoginName") is not None else None
        })
        return _obj


